/*
 * EvolutionaryAlgorithmC.h - Simple evolutionary algorithm.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 23-Jun-1998
 *
 */

#ifndef EvolutionaryAlgorithmC_h
#define EvolutionaryAlgorithmC_h 1

#include "Core/EvolutionaryAlgorithmA.h"

#include "Core/PopulationC.h"
#include "Core/StopConditionC.h"
#include "StatisticsC.h"

#include "Core/CoreFactoryA.h"


/*
 * This class shows how to implement a simple evolutionary algorithm with one population and a stopcondition. The stopcondition is given to other classes in the library. When such a class has a reason to terminate the run, for example when a solution has been found or when a maximum number of evaluation has been reached, it can ask this class for termination.
 *
 * @memo Simple evolutionary algorithm
 *
 * @see EvolutionaryAlgorithmA
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class EvolutionaryAlgorithmC: public EvolutionaryAlgorithmA
{
	public:

		/** 
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt EvolutionaryAlgorithmC}
		 * \subitem{\tt EvolutionaryAlgorithm(Configuration, CoreFactory, ProblemFactory, ReproductionFactory, Problem);}
		 * \end{description}
		 * This constructor is only called by the CoreFactory, and should {\em not} be called from somewhere else.
		 * If a new EvolutionaryAlgorithm class is to be used, the {\tt CoreFactoryC} should be inherited and the {\tt CreateEvolutionaryAlgorithm()} method should be overwritten. If more classes are changed it can be useful to inherit from {\tt CoreFactoryA} class itself.
		 *
		 * @memo This constructor is only called by the CoreFactory
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iStatistics The statistics are used to write data to and build a report after a run of the evolutionary algorithm.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iReproductionFactory The factory that is used to spawn objects from classes of the Reproduction package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblem Actual problem the evolutionary algorithm has to work on.
		 * @return Nothing.
		 *
		 * @see CoreFactoryA
		 * @see CoreFactoryC 
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		EvolutionaryAlgorithmC
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			ProblemA const & iProblem
		);

		virtual ~EvolutionaryAlgorithmC();


		/** 
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt EvolutionaryAlgorithmA * EvolutionaryAlgorithm}
		 * \subitem{\tt = CoreFactory->CreateEvolutionaryAlgorithm(ProblemFactory, ReproductionFactory);}
		 * \item[]{\tt EvolutionaryAlgorithm->Run();}
		 * \end{description}
		 * It runs the evolutionary algorithm until the stopcondition is true and the it collects the data from the run.
		 *
		 * @memo Running the Evolutionary Algorithm
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		virtual void Run();

	protected:

		/** This simple implementation has one population.
		 *
		 * @see StopConditionA
		 */
		PopulationA * Population;


		/** The stopcondition for requesting a termination of the run.
		 *
		 * @see PopulationA
		 */
		StopConditionA * StopCondition;

	private:

}; // EvolutionaryAlgorithmC

#endif

// Eof EvolutionaryAlgorithmC.h
