/*
 * GenotypeA.h - Abstract base class defining the interface of a genotype.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */

#ifndef GenotypeA_h
#define GenotypeA_h 1

#include "Problem/FitnessFunctionA.h"
#include "Problem/DataA.h"
#include "Core/InitializationA.h"
#include "typedef.h"

#include <fstream.h>

/** The genotype consists of a data and a fitness part. The data will be manipulated by the reproduction operators. The fitness part is used by the selection operators. Ofcourse additional ideas can be inorporated into children of this class.
 *
 * @memo The genotype
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class GenotypeA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. Its children should create the following objects: \begin{itemize} \item[]{Data --- which holds the data for reproduction} \item[]{Fitness --- used for selection purposes} \end{itemize}
		 *
		 * @memo Creating a new genotype
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		GenotypeA
		(
			ConfigurationA const & iConfiguration,
			ProblemFactoryA const & iProblemFactory
		);


		virtual ~GenotypeA();


		/** This function evaluates the data of the genotype and saves it as its fitness. It does this using the provided fitnessfunction.
		 *
		 * @memo Evaluating a genotype
		 *
		 * @param iFitnessFunction Used for calculating the fitness of the data in this genotype.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void Evaluate
		(
			FitnessFunctionA const & iFitnessFunction
		);
		

		/** This function initialises the data of the genotype using the provided initialization unit.
		 *
		 * @memo Initializing a genotype
		 *
		 * @param iInitialization Unit that fills the the data part with initial (random) data.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void Init
		(
			InitializationA const & iInitialization
		) =0;


		/** When data has to be changed/looked at by the reproduction unit, it can get a hold of it with this function.
		 *
		 * @memo Looking/changing data
		 *
		 * @return Data from genotype.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual DataA & GetData() const=0;



		/** Mostly for statistics usage this function gives a way to access the fitnessvalue of the genotype.
		 *
		 * @memo Getting the fitnessvalue
		 *
		 * @return Current fitnessvalue.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual FitnessType const & GetFitness() const =0;


		/** Every new created genotype gets a unique number, with this function this number can be viewed. Added for ease of debugging.
		 *
		 * @memo Getting a genotype's unique number
		 *
		 * @return Genotype's unique number.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		unsigned int GetUniqueID() const;


		/** 
		 * @name Comparator operators
		 *
		 * For comparing genotypes, or better said: for comparing genotype's fitness these operators are defined.
		 *
		 * @memo Comparing genotypes
		 *
		 * @return Result of operation as a boolean.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		///
		bool operator==
		(
			GenotypeA const & iGenotype
		) const;

		///
		bool operator!=
		(
			GenotypeA const & iGenotype
		) const;

		///
		bool operator<
		(
			GenotypeA const & iGenotype
		) const;

		///
		bool operator>
		(
			GenotypeA const & iGenotype
		) const;

		///
		bool operator<=
		(
			GenotypeA const & iGenotype
		) const;

		///
		bool operator>=
		(
			GenotypeA const & iGenotype
		) const;

		//@}


		friend ostream & operator<<
		(
			ostream & in,
			GenotypeA const & iGenotype
		)
		{
			iGenotype.PrintToStream(in);
			return in;	
		};

	protected:

		virtual void PrintToStream
		(
			ostream & in
		) const =0;


		ConfigurationA const & Configuration;

		ProblemFactoryA const & ProblemFactory;

		FitnessType FitnessValue;

	private:

		/// Unique number generator
		static unsigned int Unique_ID;

		/// Genotype's unique identification number
		unsigned int ID;

}; // GenotypeA

#endif

// Eof GenotypeA.h
