/*
 * StopConditionA.h - Abstract base class defining the interface of a stopcondition.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 19-Jun-1998
 *
 */

#ifndef StopConditionA_h
#define StopConditionA_h 1


/** An object that gets this class as a parameter can request a termination of the evolutionary algorithm.
 *
 * @memo Terminating the evolutionary algorithm
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class StopConditionA
{
	public:

		virtual ~StopConditionA() =0;

		/**
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt if (Statistics.GetNumberOfEvaluations() == MaximumAllowedNumberOfEvaluations)}
		 * \begin{description} \item[]{\tt StopCondition.RequestStop();} \end{description}
		 * \end{description}
		 *
		 * @memo Terminating the evolutionary algorithm
		 * 
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		virtual void RequestStop() const =0;


		/**
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt if (Genotype.GetFitness() == Optimum)}
		 * \begin{description} \item[]{\tt StopCondition.FoundSolution();} \end{description}
		 * \end{description}
		 *
		 * @memo A solution has been found
		 * 
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		virtual void FoundSolution() const =0;


		/**
		 * @name Information functions
		 *
		 * \begin{description}
		 * \item[] Examples: 
		 * \item[]{\tt while (!StopCondition->IsStopRequested())}
		 * \begin{description} \item[]  {\em #/##/# Let the evolutionary algorithm run} \end{description}
		 * \item[]{\tt if (StopCondition->IsSolutionFound());}
		 * \begin{description} \item[]{\tt cout << "Found solution" << endl;} \end{description}
		 * \end{description}
		 *
		 * @memo Checking state of stopconditon
		 * 
		 * @return Boolean value answering question.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 */

		//@{

		///
		virtual bool IsSolutionFound() const =0;

		///
		virtual bool IsStopRequested() const =0;

		//@}

	protected:

	private:

}; // StopConditionA

#endif

// Eof StopConditionA.h
