/*
 * FitnessFunctionA.h - Abstract base class defining the interface of a fitnessfunction.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */

#ifndef FitnessFunctionA_h
#define FitnessFunctionA_h 1

#include "Problem/ProblemFactoryA.h"
#include "Problem/ProblemA.h"
#include "Problem/DataA.h"
#include "Core/StopConditionA.h"
#include "typedef.h"

/** This fitnessfunction will be used by the genotype to evaluate its data and get a fitnessvalue. These fitnessvalues can then be used for ordening the genotypes in a population.
 *
 * @memo A function for awarding fitness to genotypes
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class FitnessFunctionA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters.
		 *
		 * @memo Creating a new fitnessfunction
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iStatistics The statistics are used to write data to and build a report after a run of the evolutionary algorithm.
		 * @param iProblem Actual problem the evolutionary algorithm has to work on, used for evaluating the data.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		FitnessFunctionA
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem
		): Configuration(iConfiguration), Statistics(iStatistics), StopCondition(iStopCondition), Problem(iProblem)
		{};

		virtual FitnessFunctionA::~FitnessFunctionA() =0;


		/**
		 * \begin{description} \item[] Example:  \item[]{\tt void Genotype::Evaluate(FitnessFunctionA const #&# FitnessFunction)} \item[] #{# \begin{description} \item[]{\tt FitnessFunction(GetFitness(), GetData());} \end{description} \item[] #}#  \end{description}
		 * The fitnessvalue provided is changed to reflect the fitness of the data that is provided.
		 *
		 * @memo Evaluating data
		 *
		 * @param iFitnessValue This fitnessvalue is altered to reflect the data provided.
		 * @param iData This is the data that will be evaluated.
		 * @return See iFitnessValue.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void const Calculate
		(
			FitnessType & iFitnessValue,
			DataA const & iData
		) const =0;

	protected:

		ConfigurationA const & Configuration;
		StatisticsA & Statistics;
		StopConditionA const & StopCondition;
		ProblemA const & Problem;

	private:

}; // FitnessFunctionA

#endif

// Eof FitnessFunctionA.h
