/*
 * ChangeOperatorA.h - Abstract base class defining the interface of a change operator.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 25-Jun-1998
 *
 */

#ifndef ChangeOperatorA_h
#define ChangeOperatorA_h 1

#include "ConfigurationA.h"
#include "Core/GenotypeA.h"
#include "Problem/FitnessFunctionA.h"
#include "Problem/DataA.h"
#include "Common/RandomA.h"
#include "Common/ArrayC.h"


/** This operator takes existing genotypes and changes them. No new genotypes are produced this way.
 *
 * @memo The change operator
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ChangeOperatorA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. 
		 *
		 * @memo Creating a new change operator
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iRandom The production operator will require some random numbers.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		ChangeOperatorA
		(
			ConfigurationA const & iConfiguration,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem,
			RandomA const & iRandom
		);

		virtual ~ChangeOperatorA();


		/** \begin{description} \item[]{\tt ArrayC<GenotypeA*>* Children;} \item[]{\em #/##/# Produce a list of children and store them in the {\tt Children} list} \item[]{\tt ChangeOperator(*Children);} \end{description} 
		 * Given a list of genotypes, the genotypes are changed according to some algorithm.
		 *
		 * @memo Changing offspring
		 *
		 * @param iArray This is a list of to-be-changed genotypes.
		 * @return iArray is the returned variable.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void operator()
		(
			ArrayC<GenotypeA*> const & iArray
		) const =0;
		
	protected:

		ConfigurationA const & Configuration;

		ProblemFactoryA const & ProblemFactory;

		ProblemA const & Problem;

		RandomA const & Random;

	private:

}; // ChangeOperatorA

#endif

// Eof ChangeOperatorA.h
