/*
 * StatisticsA.h - A base class for statistics.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 17-Jun-1998
 *
 */

//TODO: Add GetNumberOfRuns and change it in the Experiment classes.

#ifndef StatisticsA_h
#define StatisticsA_h 1

#include "ConfigurationA.h"
#include "typedef.h"

#include <iostream.h>
#include <iomanip.h>


/** Abstract base class defining the basic statistical data to be collected during experiments. These consist of {\em SuccessRate} (SR), {\em Average Evaluations to Solution} (AES) and {\em Mean Fitness} (MF).
 *
 * @memo Abstract statistics class
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class StatisticsA 
{
	public:

		StatisticsA
		(
			ConfigurationA const & iConfiguration
		): Configuration(iConfiguration), NumberOfRuns(0), TotalFitness(0), TotalSuccess(0), TotalNumberOfEvaluations(0)
		{};

		virtual ~StatisticsA() =0;


		/**
		 * \begin{description}
		 * \item[] Example:
		 * \item[]{\tt Statistics.ResetRunStatistics();}
		 * \item[]{\em #/##/# Do some runs of the evolutionary algorithm on a problem.}
		 * \item[]{\tt cout << Statistics;}
		 * \end{description}
		 * Call this to reset all the statistics that are used to collect data during runs.
		 *
		 * @memo Reseting statistics
		 *
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void ResetRunStatistics();


		/**
		 * \begin{description}
		 * \item[] Example:
		 * \item[]{\tt Statistics.ResetRunStatistics();}
		 * \item[]{\tt for (runs = 0; runs < 10; ++runs)}
		 * \item[] #{#
		 * \begin{description} \item[]{\em #/##/# Run the evolutionary algorithm.}
		 * \item[]{\tt Statistics.CollectBasicRunStatistics(SolutionFound, BestIndividual->GetFitness());}
		 * \end{description}
		 * \item[] #}#
		 * \item[]{\tt cout << Statistics;}
		 * \end{description}
		 * After every run this function should be called to record the result of the last run.
		 *
		 * @memo Collecting statistics
		 *
		 * @param iSuccess Set this to {\tt true} if the last run found a solution.
		 * @param iBestFitness This should be the fitness of the best individual in population after the last run.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void CollectBasicRunStatistics
		(
			bool iIsSucces,
			FitnessType const & iBestFitness
		) =0;


		/**
		 * @name Registering Evaluations
		 *
		 * \begin{description}
		 * \item[] Example:
		 * \item[]{\tt FitnessFunction::Calculate();}
		 * \item[] #{#
		 * \begin{description} \item[]{\em #/##/# Calculate the fitness of some genome using its data.}
		 * \item[]{\tt Statistics.RegisterEvaluation();}
		 * \end{description}
		 * \item[] #}#
		 * \item[]{\tt cout << Statistics;}
		 * \end{description}
		 * These functions are used to register the amount of evaluations made by the fitnessfunction.
		 *
		 * @memo Collecting statistics
		 *
		 * @param iEvaluations Number of evaluations made, only needed when you want to register a number of evaluations in one single command.
		 * @return Nothing.
		 *
		 * @see FitnessFunctionA
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		/// Single evaluation.
		virtual void RegisterEvaluation() =0;

		/// Multiple evaluations,
		virtual void RegisterEvaluations
		(
			unsigned long int iEvaluations
		) =0;
		//@}

		
		/**
		 * This function can be used in combination with the StopCondition. When a maximum number of evaluations is allowed, StopCondition can be used to terminate the current run when this number is reached.
		 *
		 * @memo Collecting statistics
		 *
		 * @return The number of evaluation done this run so far.
		 *
		 * @see StopConditionA
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual unsigned long int GetNumberOfEvaluations() const =0;

		friend ostream & operator<<
		(
			ostream & os,
			StatisticsA const & iStatistics
		)
		{
			iStatistics.PrintToStream(os);

			return os;
		};

	protected:

		ConfigurationA const & Configuration;

		unsigned long int NumberOfRuns;

		/// Total of best fitnesses this run.
		double TotalFitness;

		/// Total number of successes this run.
		unsigned long int TotalSuccess;

		/// Number of evaluations this run.
		unsigned long int NumberOfEvaluations;

		/// Number of evaluations over all runs.
		unsigned long int TotalNumberOfEvaluations;

		virtual void PrintToStream
		(
			ostream & iOstream
		) const =0;

	private:

}; // StatisticsA

#endif

// Eof StatisticsA.h
