/*
 * ReproductionC.cc - A simple implementation of a reproduction unit.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#include "Reproduction/ReproductionC.h"
#include "Common/ArrayC.tpl"


ReproductionC::ReproductionC
(
	ConfigurationA const & iConfiguration,
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	StopConditionA const & iStopCondition,
	ProblemA const &iProblem,
	RandomA const & iRandom
): ReproductionA(iConfiguration, iCoreFactory, iProblemFactory, iReproductionFactory, iStopCondition, iProblem, iRandom)
{
	NumberOfOperationsPerGeneration = (Configuration)("Reproduction", "NumberOfOperationsPerGeneration");

	ProduceOperator = ReproductionFactory.CreateProduceOperator(CoreFactory, ProblemFactory, Problem);
	ChangeOperator = ReproductionFactory.CreateChangeOperator(ProblemFactory, Problem);
	SelectParents = ReproductionFactory.CreateSelectParents();
	SelectSurvivors = ReproductionFactory.CreateSelectSurvivors();

} // ReproductionC


ReproductionC::~ReproductionC()
{
	delete ProduceOperator;
	delete ChangeOperator;
	delete SelectParents;
	delete SelectSurvivors;
}  // ~ReproductionC



void
ReproductionC::operator()
(
	PoolA & iPool,
	FitnessFunctionA const & iFitnessFunction,
	StatisticsA & iStatistics
)
{
	ArrayC<GenotypeA const *>* Parents;
	ArrayC<GenotypeA*>* Children;

	for (unsigned int i = 0; i < NumberOfOperationsPerGeneration; ++i)
	{

		// First select some parents for this operation.
		Parents = (*SelectParents)(iPool);

		// Now breed new offspring and mutate the offspring.
		Children = (*ProduceOperator)(*Parents);
		(*ChangeOperator)(*Children);

		// Evaluate the newborns.
		for (unsigned int j = 0; j < Children->GetSize(); ++j)
		{
			//iStatistics.RegisterEvaluation;
			(*Children)[j]->Evaluate(iFitnessFunction);
		}

		// Kill of the badguys.
		(*SelectSurvivors)(iPool, *Children);

		// Free used containers.
		delete Children;
		delete Parents;
	}

	// Are we over the limit of the number of allowed generations?,D
	if (iStatistics.GetNumberOfEvaluations() >= Configuration("Reproduction", "MaximumNumberOfOperations"))
		StopCondition.RequestStop();

} // operator()



// Eof ReproductionC.cc
